#!/usr/bin/env pybricks-micropython

from pybricks.hubs import EV3Brick
from pybricks.ev3devices import (Motor, TouchSensor, ColorSensor,
                                 InfraredSensor, UltrasonicSensor, GyroSensor)
from pybricks.parameters import Port, Stop, Direction, Button, Color
from pybricks.tools import wait, StopWatch, DataLog
from pybricks.robotics import DriveBase
from pybricks.media.ev3dev import SoundFile, ImageFile
import time

S_timer = time.time()

# This program requires LEGO EV3 MicroPython v2.0 or higher.
# Click "Open user guide" on the EV3 extension tab for more information.


# Create your objects here.
ev3 = EV3Brick()
left_motor = Motor(Port.B, Direction.COUNTERCLOCKWISE, [12, 36])
right_motor = Motor(Port.C, Direction.CLOCKWISE, [12, 36])
left_sensor = ColorSensor(Port.S2)
advanced_sensor = ColorSensor(Port.S4)
right_sensor = ColorSensor(Port.S3)
I_see_you = UltrasonicSensor(Port.S1)
advance_motor = Motor(Port.D)
robot = DriveBase(left_motor, right_motor, wheel_diameter=80, axle_track=220)

ev3.speaker.beep()
while not(Button.CENTER in ev3.buttons.pressed()):
    wait(10)

#Вводим переменную скорости движения
DRIVE_SPEED = 50

#Водим переменную пропорциональности(нужно для движения по линии)
PROPORTIONAL_GAIN = 1.1
RG = 0
LG = 0
lt = 0

# Write your program here.

advance_motor.run(-10)

while left_sensor.reflection() < 95 and left_sensor.reflection() < 95:
    turn_rate = (left_sensor.reflection() - right_sensor.reflection())*PROPORTIONAL_GAIN
    robot.drive(DRIVE_SPEED, turn_rate)

    if left_sensor.reflection() < 20 and right_sensor.reflection() < 20: #Если перекрёсток, то проедь прямо
        robot.straight(200)
    
    if I_see_you.distance() < 100:
        robot.turn(-90)
        br = 0
        robot.straight(100)
        robot.turn(90)
        right_motor.reset_angle()
        while right_motor.angle() < 360:
            robot.drive(50, 0)
            if right_sensor.reflection() < 20:
                robot.stop()
                break
                br = 1
        robot.turn(90)
        while right_motor.angle() < 287 and br == 0:
            robot.drive(50, 0)
            if right_sensor.reflection() < 20:
                robot.stop()
                break
                br = 1
        robot.turn(90)
        while right_motor.angle() < 360 and br == 0:
            robot.drive(50, 0)
            if right_sensor.reflection() < 20:
                robot.stop()
                break
        robot.turn(-30)


    if left_sensor.color() == Color.GREEN or right_sensor.color() == Color.GREEN: #Зелёный индикатор
        robot.stop()
        ev3.speaker.beep()
        if left_sensor.color() == Color.GREEN and right_sensor.color() == Color.GREEN:
            RG = 1
            LG = 1
        elif left_sensor.color() == Color.GREEN:
            LG = 1
            robot.turn(1)
            if right_sensor.color() == Color.GREEN:
                RG = 1
        elif right_sensor.color() == Color.GREEN:
            RG = 1
            robot.turn(-1)
            if left_sensor.color() == Color.GREEN:
                LG = 1
        if RG == LG and RG == 1:
            RG = 0
            LG = 0
            robot.turn(135)
            while right_sensor.reflection() > 20:
                robot.drive(50, 90)
        elif RG == 1:
            RG = 0
            robot.turn(70)
            while left_sensor.reflection() > 20:
                robot.drive(50, 90)
        elif LG == 1:
            LG = 0
            robot.turn(-70)
            while right_sensor.reflection() > 20:
                robot.drive(50, -90)

    if left_sensor.reflection() > 20 and right_sensor.reflection() > 20:
        if time.time() - S_timer > 1:
            ev3.speaker.beep()
            S_timer = time.time()
            if right_sensor.reflection() > 20 and left_sensor.reflection() > 20:
                robot.stop()
                ev3.speaker.beep()
                g = 40
                k = 1 * lt
                for j in range(4):
                    robot.drive(20, 90*k)
                    S_timer = time.time()
                    while right_sensor.reflection() > 20 and left_sensor.reflection() > 20 and time.time() - S_timer < 2*(j+1):
                        robot.drive(25, 90*k)
                    k *= -1
    else:
        if right_sensor.reflection() < 20:
            lt = 1
        elif left_sensor.reflection() < 20:
            lt = -1
        S_timer = time.time()

robot.stop

#Собираем шары
robot.straight(225)
robot.straight(200)
robot.straight(-200)
advance_motor.run_time(50, 3000, then=Stop.COAST)
robot.turn(90)
for i in range(4):
    robot.drive(50, 0)
    wait(3000)
    if I_see_you.distance() > 90:
        NoA = i
        robot.turn(-45)
        robot.drive(50, 0)
        wait(1000)
        robot.turn(-44)
    else:
        advance_motor.run_time(-50, 3000)
        robot.turn(-85)
        advance_motor.run_time(50, 3000, then=Stop.COAST)
robot.turn(-90)
robot.drive(50, 0)
wait(3000)
k = -75
for i in range(1, 7):
    for j in range(4):
        robot.drive(50, 0)
        wait(3000)
        if j == NoA:
            if I_see_you.distance() > 90:
                robot.turn(-45)
                robot.drive(50, 0)
                wait(1000)
                robot.turn(-44)
            else:
                advance_motor.run_time(-50, 3000)
                robot.turn(-90)
                advance_motor.run_time(50, 3000, then=Stop.COAST)
        else:
            advance_motor.run_time(-50, 3000)
            advance_motor.run_time(50, 3000, then=Stop.COAST)
            robot.straight(k*i)
            robot.turn(-90)
robot.straight(-450)

if NoA == 3:
    robot.turn(30)
elif NoA==2:
    robot.turn(120)
elif NoA == 4:
    robot.turn(-30)
elif NoA == 1:
    robot.turn(-120)

robot.drive(-50, 0)
wait(3000)

#сброс жертв
#....

robot.straight(500)

while I_see_you.distance() < 180:
    robot.drive(50, 90)
robot.turn(2)
while left_sensor.color() == Color.GREEN:
    robot.drive(50, 0)
robot.straight(100)

while left_sensor.color() == Color.RED and left_sensor.color() == Color.RED:
    turn_rate = (left_sensor.reflection() - right_sensor.reflection())*PROPORTIONAL_GAIN
    robot.drive(DRIVE_SPEED, turn_rate)

    if left_sensor.reflection() < 20 and right_sensor.reflection() < 20: #Если перекрёсток, то проедь прямо
        robot.straight(200)
    
    if I_see_you.distance() < 100:
        robot.turn(-90)
        br = 0
        robot.straight(100)
        robot.turn(90)
        right_motor.reset_angle()
        while right_motor.angle() < 360:
            robot.drive(50, 0)
            if right_sensor.reflection() < 20:
                robot.stop()
                break
                br = 1
        robot.turn(90)
        while right_motor.angle() < 287 and br == 0:
            robot.drive(50, 0)
            if right_sensor.reflection() < 20:
                robot.stop()
                break
                br = 1
        robot.turn(90)
        while right_motor.angle() < 360 and br == 0:
            robot.drive(50, 0)
            if right_sensor.reflection() < 20:
                robot.stop()
                break
        robot.turn(-30)


    if left_sensor.color() == Color.GREEN or right_sensor.color() == Color.GREEN: #Зелёный индикатор
        robot.stop()
        ev3.speaker.beep()
        if left_sensor.color() == Color.GREEN and right_sensor.color() == Color.GREEN:
            RG = 1
            LG = 1
        elif left_sensor.color() == Color.GREEN:
            LG = 1
            robot.turn(1)
            if right_sensor.color() == Color.GREEN:
                RG = 1
        elif right_sensor.color() == Color.GREEN:
            RG = 1
            robot.turn(-1)
            if left_sensor.color() == Color.GREEN:
                LG = 1
        if RG == LG and RG == 1:
            RG = 0
            LG = 0
            robot.turn(135)
            while right_sensor.reflection() > 20:
                robot.drive(50, 90)
        elif RG == 1:
            RG = 0
            robot.turn(70)
            while left_sensor.reflection() > 20:
                robot.drive(50, 90)
        elif LG == 1:
            LG = 0
            robot.turn(-70)
            while right_sensor.reflection() > 20:
                robot.drive(50, -90)

    if left_sensor.reflection() > 20 and right_sensor.reflection() > 20:
        if time.time() - S_timer > 1:
            ev3.speaker.beep()
            S_timer = time.time()
            if right_sensor.reflection() > 20 and left_sensor.reflection() > 20:
                robot.stop()
                ev3.speaker.beep()
                g = 40
                k = 1 * lt
                for j in range(4):
                    for i in range(g):
                        if right_sensor.reflection() > 20 and left_sensor.reflection() > 20:
                            robot.turn(k)
                        else:
                            k = 2
                            break
                    if k == 2:
                        break
                    else:
                        g = g + 40
                        k = k * (-1)
                if k != 2:
                    robot.turn(80)
    else:
        if right_sensor.reflection() < 20:
            lt = 1
        elif left_sensor.reflection() < 20:
            lt = -1
        S_timer = time.time()

robot.stop(63)
